using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using GroupLab.GTNetworking;
using EasyImages;
using System.Net;
using System.IO;

namespace SDFTPUploader
{
    public partial class Form1 : Form
    {
        private EasyImages.CameraClient camera;
        bool ftpOn;
        int writeCounter = 0;
        int otherCounter = 0;
        private delegate void SetPictureBoxImage(PictureBox pbox, Bitmap image);
        ftpUpdater fup;
        public Form1()
        {
            InitializeComponent();
            ftpOn = false;
        }

        private void subscr_notified(object sender, SubscriptionEventArgs args)
        {
            if (ftpOn)
            {
                otherCounter++;
                writeCounter++;
                EasyImages.JPEG val = (EasyImages.JPEG)args.Value;
                if (writeCounter > 30)
                {
                    writeCounter = 0;
                    bool worked = fup.Upload(val);
                    fup.Upload(otherCounter.ToString());
                    if (!worked)
                    {
                        
                        ftpLogOn.Enabled = true;
                        ftpLogOn.Text = "Retry";
                        ftpOn = false;
                    }
                    else ftpLogOn.Text = "Uploading";
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            camera = new EasyImages.CameraClient("DefaultCamera");
            camera.FramesPerSecond = 1;
            camera.ReceivedFrame += new CameraClient.ReceivedFrameEventHandler(camera_ReceivedFrame);
            camera.Start();
        }

        private void ftpLogOn_Click(object sender, EventArgs e)
        {
            fup = new ftpUpdater(ftpServer.Text, ftpLogin.Text, ftpPass.Text);
            ftpLogOn.Enabled = false;
            ftpLogOn.Text = "Attempting";
            ftpOn = true;
        }

        private void SD_Opened(object sender, System.EventArgs args)
        {
            // make a subscription to /message so we get notified of incoming messages
            this.subscription1.BeginInit();
            this.subscription1.Dictionary = this.sharedDictionary;
            this.subscription1.Pattern = "/imageToSend";
            // set the event handler in case a message arives
            this.subscription1.Notified += new SubscriptionEventHandler(this.subscr_notified);
            this.subscription1.EndInit();

            // let the user send messages to the server and see incoming messages

            this.sdButton.Text = "Disconnect";
            this.sdButton.Enabled = true;
        }

        private void SD_Closed(object sender, System.EventArgs args)
        {
            this.sdButton.Text = "Connect";
            this.sdButton.Enabled = true;// show the connect button so the user can retry
        }

        private void sdButton_Click(object sender, EventArgs e)
        {
            if (this.sdButton.Text == "Connect!")
            {
                string url = "tcp://localhost:hello";

                if (DialogResult.OK == GroupLab.GTNetworking.UrlPrompt.Show(ref url))
                {
                    this.sharedDictionary.Url = url;
                    // we want be notified when we're connected or disconnected
                    this.sharedDictionary.Opened += new EventHandler(SD_Opened);
                    this.sharedDictionary.Closed += new EventHandler(SD_Closed);
                    this.sharedDictionary.Open(); // start a server or client dictionary
                    this.sdButton.Enabled = false;
                }
            }
            else
            {
                this.sharedDictionary.Close(); // close the connection with the shared dictionary
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.sharedDictionary.Close();
            base.OnClosing(e);
        }

        private void camera_ReceivedFrame(object sender, CameraEventArgs e)
        {
            this.DisplayImageInPictureBox(pictureBoxVideo, e.Bitmap);
            if (sharedDictionary.Status == ConnectionStatus.Opened)
            {
                sharedDictionary["/imageToSend"] = new EasyImages.JPEG(e.Bitmap);
            }
        }

        private void DisplayImageInPictureBox(PictureBox pbox, Image image)
        {
            if (pbox.InvokeRequired) // We are in the wrong thread. Call ourselves in the correct thread
            {
                SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
                BeginInvoke(theDelegate, new object[] { pbox, image });
            }
            else // we are in the correct thread, so assign the image
            {
                pbox.Image = image;
            }
        }
    }

    public class ftpUpdater
    {
        string ftpServer, ftpUserID, ftpPassword;
        public ftpUpdater(string newFtpServer, string newFtpUserID, string newFtpPassword)
        {
            ftpServer = newFtpServer;
            ftpUserID = newFtpUserID;
            ftpPassword = newFtpPassword;
        }

        public bool Upload(string newMessage)
        {
            System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
            byte[] textBytes = encoding.GetBytes(newMessage);

            string uri = "ftp://" + ftpServer + "/" + "uploadedText.txt";
            FtpWebRequest reqFTP;

            reqFTP = (FtpWebRequest)FtpWebRequest.Create(new Uri(uri));
            reqFTP.Credentials = new NetworkCredential(ftpUserID, ftpPassword);
            reqFTP.KeepAlive = false;
            reqFTP.Method = WebRequestMethods.Ftp.UploadFile;
            reqFTP.UseBinary = true;
            reqFTP.ContentLength = textBytes.Length;
            try
            {
                Stream strm = reqFTP.GetRequestStream();
                strm.Write(textBytes, 0, textBytes.Length);
                strm.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Upload Error");
                return false;
            }
            return true;
        }

        public bool Upload(EasyImages.JPEG image)
        {
            byte[] imageBytes = image.GetByteArray();
            string uri = "ftp://" + ftpServer + "/" + "uploadedImage.jpg";
            FtpWebRequest reqFTP;

            reqFTP = (FtpWebRequest)FtpWebRequest.Create(new Uri(uri));
            reqFTP.Credentials = new NetworkCredential(ftpUserID, ftpPassword);
            reqFTP.KeepAlive = false;
            reqFTP.Method = WebRequestMethods.Ftp.UploadFile;
            reqFTP.UseBinary = true;
            reqFTP.ContentLength = imageBytes.Length;

            int buffLength = 2048;
            byte[] buff = new byte[buffLength];

            try
            {
                Stream strm = reqFTP.GetRequestStream();
                strm.Write(imageBytes, 0, imageBytes.Length);
                strm.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Upload Error");
                return false;
            }
            return true;
        }
    }
}